import logging
from typing import List

from shikimori.types.animes import Anime
from shikimori.types.club import Club, ClubImage, ClubInfo, Collection
from shikimori.types.manga import Manga
from shikimori.types.ranobe import Ranobe
from shikimori.types.character import CharacterBrief
from .base import BaseEndpoint
from ..exceptions import RequestError
from ..types.user import User
from ..utils.filter import filter_none_parameters

logger = logging.getLogger(__name__)


class ClubEndpoint(BaseEndpoint):
    async def list(
        self, page: int = None, limit: int = None, search: str = None
    ) -> List[Club] | RequestError:
        """
        List clubs.

        :param page: Must be a number between 1 and 100000.
        :param limit: 30 max.
        :param search: string.
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/clubs",
            headers=self.headers,
            params=filter_none_parameters(
                {"limit": limit, "page": page, "search": search}
            ),
        )

        if not isinstance(response, RequestError):
            return [Club.from_dict(club) for club in response]

        logger.debug(
            f"Bad Request(list): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def ById(self, id: int) -> ClubInfo | RequestError:
        """
        Show a club.

        :param id: must be a number.
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/clubs/{id}",
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return ClubInfo.from_dict(response)

        logger.debug(
            f"Bad Request(ById): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def update(
        self,
        id: int,
        name: str = None,
        description: str = None,
        display_images: bool | int = None,
        comment_policy: str = None,
        topic_policy: str = None,
        image_upload_policy: str = None,
    ) -> ClubInfo | RequestError:
        """
        Update a club.

        Requires clubs oauth scope.

        :param id: number.
        :param name: string.
        :param description: string.
        :param display_images: Must be one of: true, false, 1, 0.
        :param comment_policy: Must be one of: free, members, admins.
        :param topic_policy: Must be one of: members, admins.
        :param image_upload_policy: Must be one of: members, admins.
        """
        response = await self._request.make_request(
            "PATCH",
            url=f"{self._base_url}/api/clubs/{id}",
            json={
                "club": filter_none_parameters(
                    {
                        "description": description,
                        "name": name,
                        "display_images": display_images,
                        "comment_policy": comment_policy,
                        "topic_policy": topic_policy,
                        "image_upload_policy": image_upload_policy,
                    }
                )
            },
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return ClubInfo.from_dict(response)

        logger.debug(
            f"Bad Request(update): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def animes(
        self, id: int, limit: int = None, page: int = None
    ) -> List[Anime] | RequestError:
        """
        List of club animes.

        :param id: must a number.
        :param limit: 20 maximum.
        :param page: Must be a number between 1 and 100000.
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/clubs/{id}/animes",
            headers=self.headers,
            params=filter_none_parameters({"limit": limit, "page": page}),
        )

        if not isinstance(response, RequestError):
            return [Anime.from_dict(anime) for anime in response]

        logger.debug(
            f"Bad Request(GetAnimes): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def mangas(
        self, id: int, page: int = None, limit: int = None
    ) -> List[Manga] | RequestError:
        """
        List of club mangas.

        :param id: must a number.
        :param limit: 20 maximum.
        :param page: Must be a number between 1 and 100000.
        """

        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/clubs/{id}/mangas",
            headers=self.headers,
            params=filter_none_parameters({"limit": limit, "page": page}),
        )

        if not isinstance(response, RequestError):
            return [Manga.from_dict(manga) for manga in response]

        logger.debug(
            f"Bad Request(GetMangas): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def ranobes(
        self, id: int, limit: int = None, page: int = None
    ) -> List[Ranobe] | RequestError:
        """
        List of club Ranobes

        :param id: must a number.
        :param limit: 20 maximum.
        :param page: Must be a number between 1 and 100000.
        """

        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/clubs/{id}/ranobe",
            headers=self.headers,
            params=filter_none_parameters({"limit": limit, "page": page}),
        )

        if not isinstance(response, RequestError):
            return [Ranobe.from_dict(ranobe) for ranobe in response]

        logger.debug(
            f"Bad Request(GetRanobes): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def characters(
        self, id: int, limit: int = None, page: int = None
    ) -> List[CharacterBrief] | RequestError:
        """
        List of club Characters.

        :param id: must a number.
        :param limit: 20 maximum.
        :param page: Must be a number between 1 and 100000.
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/clubs/{id}/characters",
            headers=self.headers,
            params=filter_none_parameters({"limit": limit, "page": page}),
        )

        if not isinstance(response, RequestError):
            return [CharacterBrief.from_dict(ch) for ch in response]

        logger.debug(
            f"Bad Request(GetCharacters): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def collections(
        self, id: int, limit: int = None, page: int = None
    ) -> List[Collection] | RequestError:
        """
        List of club Collections.

        :param id: must a number.
        :param limit: 4 maximum.
        :param page: Must be a number between 1 and 100000.
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/clubs/{id}/collections",
            headers=self.headers,
            params=filter_none_parameters({"limit": limit, "page": page}),
        )

        if not isinstance(response, RequestError):
            return [Collection.from_dict(coll) for coll in response]

        logger.debug(
            f"Bad Request(GetCollections): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def clubs(
        self, id: int, limit: int = None, page: int = None
    ) -> List[Club] | RequestError:
        """
        List of club clubs.

        :param id: must a number.
        :param limit: 30 maximum.
        :param page: Must be a number between 1 and 100000.
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/clubs/{id}/clubs",
            headers=self.headers,
            params=filter_none_parameters({"limit": limit, "page": page}),
        )

        if not isinstance(response, RequestError):
            return [Club.from_dict(c) for c in response]

        logger.debug(
            f"Bad Request(Clubs): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def members(
        self, id: int, limit: int = None, page: int = None
    ) -> List[User] | RequestError:
        """
        List of club members.

        :param id: must a number.
        :param limit: 100 maximum.
        :param page: Must be a number between 1 and 100000.
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/clubs/{id}/members",
            headers=self.headers,
            params=filter_none_parameters({"limit": limit, "page": page}),
        )

        if not isinstance(response, RequestError):
            return [User.from_dict(u) for u in response]

        logger.debug(
            f"Bad Request(Members): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def images(
        self, id: int, limit: int = None, page: int = None
    ) -> List[ClubImage] | RequestError:
        """
        List of club Images.

        :param id: must a number.
        :param limit: 100 maximum.
        :param page: Must be a number between 1 and 100000.
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/clubs/{id}/images",
            headers=self.headers,
            params=filter_none_parameters({"limit": limit, "page": page}),
        )

        if not isinstance(response, RequestError):
            return [ClubImage.from_dict(s) for s in response]

        logger.debug(
            f"Bad Request(Images): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def join(self, id: int) -> None | RequestError:
        """
        Join a club.

        Requires clubs oauth scope.

        :param id: must be a number.
        """
        response = await self._request.make_request(
            "POST",
            url=f"{self._base_url}/api/clubs/{id}/join",
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return

        logger.debug(
            f"Bad Request(Join): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def leave(self, id: int) -> None | RequestError:
        """
        Leave a club.

        Requires clubs oauth scope.

        :param id: must be a number.
        """
        response = await self._request.make_request(
            "POST",
            url=f"{self._base_url}/api/clubs/{id}/leave",
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return

        logger.debug(
            f"Bad Request(Leave): status - {response.status_code}: info - {str(response)}"
        )

        return response



import logging

from shikimori.enums import (
    OrderEnum,
    GenreEntryTypeEnum,
    UserRateTargetTypeEnum,
    UserRateStatusEnum,
    UserRateOrderInputType,
)
from shikimori.exceptions import RequestError
from .base import BaseEndpoint

logger = logging.getLogger(__name__)


class GraphQlEndpoint(BaseEndpoint):
    async def animes(
        self,
        fields: str,
        page: int = 1,
        limit: int = 1,
        order: OrderEnum = None,
        kind: str = None,
        status: str = None,
        season: str = None,
        score: int = None,
        duration: int = None,
        rating: str = None,
        genre: str = None,
        studio: str = None,
        franchise: str = None,
        censored=True,
        mylist: str = None,
        ids: str = None,
        excludeIds: str = None,
        search: str = None,
    ) -> dict | RequestError:
        """

        :param fields: e.g. '{ anime { id name } }'  (check more in doc https://shikimori.one/api/doc/graphql )
        :param kind: List of values separated by comma. Add ! before value to apply negative filter. values: tv, movie, ova, ona, special, tv_special, music, pv, cm, tv_13, tv_24, tv_48
        :param status: List of values separated by comma. Add ! before value to apply negative filter.
        :param season: List of values separated by comma. Add ! before value to apply negative filter. Examples: summer_2017, 2016, 2014_2016, 199x
        :param score: Minimal anime score
        :param duration: List of values separated by comma. Add ! before value to apply negative filter. Examples: S - Less than 10 minutes, D - Less than 30 minutes, F - More than 30 minutes
        :param rating: List of values separated by comma. Add ! before value to apply negative filter. Examples: none - No rating, g - G - All ages, pg - PG - Children, pg_13 - PG-13 - Teens 13 or older, r - R - 17+ recommended (violence & profanity), r_plus - R+ - Mild Nudity (may also contain violence & profanity), rx - Rx - Hentai (extreme sexual content/nudity)
        :param genre: List of comma separated genre ids
        :param studio: List of comma separated studio ids
        :param franchise: List of comma separated franchises
        :param censored: Set to false to allow hentai, yaoi and yuri
        :param mylist: List of values separated by comma. Add ! before value to apply negative filter. Examples: planned - Planned to Watch, watching - Watching, rewatching - Rewatching, completed - Completed, on_hold - On Hold, dropped - Dropped
        :param ids: List of comma separated ids
        :param excludeIds: List of comma separated ids
        :param search: must be a string
        :param order: OrderEnum
        :param page: must be a number
        :param limit: Maximum 50
        """

        query = (
            """query($page: Int, $limit: Int, $order: OrderEnum, $kind: AnimeKindString, $status: AnimeStatusString, $season: SeasonString, $score: Int, $duration: DurationString, $rating: RatingString, $genre: String, $studio: String, $franchise: String, $censored: Boolean, $mylist: MylistString, $ids: String, $excludeIds: String, $search: String){
        animes(
            page: $page,
            limit: $limit,
            order: $order,
            kind: $kind,
            status: $status,
            season: $season,
            score: $score,
            duration: $duration,
            rating: $rating,
            genre: $genre,
            studio: $studio,
            franchise: $franchise,
            censored: $censored,
            mylist: $mylist,
            ids: $ids,
            excludeIds: $excludeIds,
            search: $search
        )"""
            + fields
            + "}"
        )

        response = await self._request.make_request(
            "POST",
            url=f"{self._base_url}/api/graphql",
            json={
                "query": query,
                "variables": {
                    "page": page,
                    "limit": limit,
                    "order": order.value if order else None,
                    "kind": kind,
                    "status": status,
                    "season": season,
                    "score": score,
                    "duration": duration,
                    "rating": rating,
                    "genre": genre,
                    "studio": studio,
                    "franchise": franchise,
                    "censored": censored,
                    "mylist": mylist,
                    "ids": ids,
                    "excludeIds": excludeIds,
                    "search": search,
                },
            },
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return response

        logger.debug(
            f"Bad Request(animes graphql): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def characters(
        self,
        fields: str,
        page: int = 1,
        limit: int = 1,
        ids: str = None,
        search: str = None,
    ) -> dict | RequestError:
        """

        :param fields: e.g. '{ anime { id name } }'  (check more in doc https://shikimori.one/api/doc/graphql )
        :param page: int
        :param limit: Maximum 50
        :param ids: list of separated ids
        :param search: str
        """

        query = (
            """
        query($page: Int, $limit: Int, $ids: [ID!], $search: String) {
          characters(
            page: $page,
            limit: $limit,
            ids: $ids,
            search: $search
          )
        """
            + fields
            + "}"
        )
        response = await self._request.make_request(
            "POST",
            url=f"{self._base_url}/api/graphql",
            json={
                "query": query,
                "variables": {
                    "page": page,
                    "limit": limit,
                    "ids": ids,
                    "search": search,
                },
            },
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return response

        logger.debug(
            f"Bad Request(characters graphql): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def contests(
        self, fields: str, page: int = 1, limit: int = 1, ids: str = None
    ) -> dict | RequestError:
        """
        :param fields: e.g. '{ anime { id name } }'  (check more in doc https://shikimori.one/api/doc/graphql )
        :param page: int
        :param limit: Maximum 10
        :param ids: list of separated ids
        """

        query = (
            """
        query($page: Int, $limit: Int, $ids: [ID!]) {
          contests(
            page: $page,
            limit: $limit,
            ids: $ids
            )
            """
            + fields
            + "}"
        )

        response = await self._request.make_request(
            "POST",
            url=f"{self._base_url}/api/graphql",
            json={
                "query": query,
                "variables": {
                    "page": page,
                    "limit": limit,
                    "ids": ids,
                },
            },
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return response

        logger.debug(
            f"Bad Request(contests graphql): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def currentUser(
        self,
        fields: str,
    ) -> dict | RequestError:
        """
        :param fields: e.g. '{ anime { id name } }'  (check more in doc https://shikimori.one/api/doc/graphql )
        """
        query = (
            """
        query {}
        """
            + fields
        )
        response = await self._request.make_request(
            "POST",
            url=f"{self._base_url}/api/graphql",
            json={
                "query": query,
            },
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return response

        logger.debug(
            f"Bad Request(currentUser graphql): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def genres(
        self, fields: str, entry: GenreEntryTypeEnum
    ) -> dict | RequestError:
        """
        :param fields: e.g. '{ anime { id name } }'  (check more in doc https://shikimori.one/api/doc/graphql )
        :param entry: GenreEntryTypeEnum
        """

        query = (
            """
        query ($entryType: GenreEntryTypeEnum!) {
          genres(entryType: $entryType)        
        """
            + fields
            + "}"
        )

        response = await self._request.make_request(
            "POST",
            url=f"{self._base_url}/api/graphql",
            json={
                "query": query,
                "variables": {
                    "entryType": str(entry),
                },
            },
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return response

        logger.debug(
            f"Bad Request(genres graphql): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def mangas(
        self,
        fields: str,
        page: int = 1,
        limit: int = 1,
        order: OrderEnum = None,
        kind: str = None,
        status: str = None,
        season: str = None,
        score: int = None,
        genre: str = None,
        publisher: str = None,
        franchise: str = None,
        censored=True,
        mylist: str = None,
        ids: str = None,
        excludeIds: str = None,
        search: str = None,
    ) -> dict | RequestError:
        """
        :param fields: e.g. '{ anime { id name } }'  (check more in doc https://shikimori.one/api/doc/graphql )
        :param publisher: List of comma separated publisher ids
        :param kind: List of values separated by comma. Add ! before value to apply negative filter. values: tv, movie, ova, ona, special, tv_special, music, pv, cm, tv_13, tv_24, tv_48
        :param status: List of values separated by comma. Add ! before value to apply negative filter.
        :param season: List of values separated by comma. Add ! before value to apply negative filter. Examples: summer_2017, 2016, 2014_2016, 199x
        :param score: Minimal anime score
        :param genre: List of comma separated genre ids
        :param franchise: List of comma separated franchises
        :param censored: Set to false to allow hentai, yaoi and yuri
        :param mylist: List of values separated by comma. Add ! before value to apply negative filter. Examples: planned - Planned to Watch, watching - Watching, rewatching - Rewatching, completed - Completed, on_hold - On Hold, dropped - Dropped
        :param ids: List of comma separated ids
        :param excludeIds: List of comma separated ids
        :param search: must be a string
        :param order: OrderEnum
        :param page: must be a number
        :param limit: Maximum 50
        """

        query = (
            """
        query ($page: Int, $limit: Int, $order: OrderEnum, $kind: MangaKindString, $status: MangaStatusString, $season: SeasonString, $score: Int, $genre: String, $publisher: String, $franchise: String, $censored: Boolean, $mylist: MylistString, $ids: String, $excludeIds: String, $search: String) {
          mangas(
            page: $page,
            limit: $limit,
            order: $order,
            kind: $kind,
            status: $status,
            season: $season,
            score: $score,
            genre: $genre,
            publisher: $publisher,
            franchise: $franchise,
            censored: $censored,
            mylist: $mylist,
            ids: $ids,
            excludeIds: $excludeIds,
            search: $search
          )
        """
            + fields
            + "}"
        )

        response = await self._request.make_request(
            "POST",
            url=f"{self._base_url}/api/graphql",
            json={
                "query": query,
                "variables": {
                    "page": page,
                    "limit": limit,
                    "order": order.value if order else None,
                    "kind": kind,
                    "status": status,
                    "season": season,
                    "score": score,
                    "genre": genre,
                    "franchise": franchise,
                    "censored": censored,
                    "mylist": mylist,
                    "ids": ids,
                    "excludeIds": excludeIds,
                    "search": search,
                    "publisher": publisher,
                },
            },
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return response

        logger.debug(
            f"Bad Request(mangas graphql): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def people(
        self,
        fields: str,
        page: int = 1,
        limit: int = 1,
        ids: str = None,
        search: str = None,
        isSeyu: bool = None,
        isProducer: bool = None,
        isMangaka: bool = None,
    ) -> dict | RequestError:
        """
        :param fields: e.g. '{ anime { id name } }'  (check more in doc https://shikimori.one/api/doc/graphql )
        :param page: int
        :param limit: Maximum 10
        :param ids: List of values separated by comma.
        :param search: str
        :param isSeyu: bool
        :param isProducer: bool
        :param isMangaka: bool
        """

        query = (
            """
        query ($page: Int, $limit: Int, $ids: [ID!], $search: String, $isSeyu: Boolean, $isProducer: Boolean, $isMangaka: Boolean) {
          people(
            page: $page,
            limit: $limit,
            ids: $ids,
            search: $search,
            isSeyu: $isSeyu,
            isProducer: $isProducer,
            isMangaka: $isMangaka
          )
        """
            + fields
            + "}"
        )
        response = await self._request.make_request(
            "POST",
            url=f"{self._base_url}/api/graphql",
            json={
                "query": query,
                "variables": {
                    "page": page,
                    "limit": limit,
                    "search": search,
                    "isSeyu": isSeyu,
                    "isProducer": isProducer,
                    "isMangaka": isMangaka,
                    "ids": ids,
                },
            },
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return response

        logger.debug(
            f"Bad Request(people graphql): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def userRates(
        self,
        fields: str,
        userId: int = None,
        page: int = 1,
        limit: int = 1,
        targetType: UserRateTargetTypeEnum = None,
        status: UserRateStatusEnum = None,
        order: UserRateOrderInputType = None,
    ) -> dict | RequestError:
        """
        :param fields: e.g. '{ anime { id name } }'  (check more in doc https://shikimori.one/api/doc/graphql )
        :param userId: ID of current user is used by default
        :param page: int
        :param limit: maximum - 50
        :param targetType: UserRateTargetTypeEnum
        :param status: UserRateStatusEnum
        :param order: UserRateOrderInputType
        """

        query = (
            """
        query ($page: Int, $limit: Int, $userId: ID, $targetType: UserRateTargetTypeEnum!, $status: UserRateStatusEnum, $order: UserRateOrderInputType) {
          userRates(
            page: $page,
            limit: $limit,
            userId: $userId,
            targetType: $targetType,
            status: $status,
            order: $order
          ) 
        """
            + fields
            + "}"
        )

        response = await self._request.make_request(
            "POST",
            url=f"{self._base_url}/api/graphql",
            json={
                "query": query,
                "variables": {
                    "page": page,
                    "limit": limit,
                    "status": status.value if status else None,
                    "userId": userId,
                    "targetType": targetType.value if targetType else None,
                    "order": (
                        {"field": order.field, "order": order.order} if order else None
                    ),
                },
            },
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return response

        logger.debug(
            f"Bad Request(userRates graphql): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def users(
        self,
        fields: str,
        page: int = 1,
        limit: int = 1,
        ids: str = None,
        search: str = None,
    ) -> dict | RequestError:
        """
        :param fields: e.g. '{ anime { id name } }'  (check more in doc https://shikimori.one/api/doc/graphql )
        :param page: int
        :param limit: maximum - 50
        :param ids: List of values separated by comma.
        :param search: str
        """

        query = (
            """
        query ($page: Int, $limit: Int, $ids: [ID!], $search: String) {
          users(
            page: $page,
            limit: $limit,
            ids: $ids,
            search: $search
          )
        """
            + fields
            + "}"
        )
        response = await self._request.make_request(
            "POST",
            url=f"{self._base_url}/api/graphql",
            json={
                "query": query,
                "variables": {
                    "page": page,
                    "limit": limit,
                    "search": search,
                    "ids": ids,
                },
            },
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return response

        logger.debug(
            f"Bad Request(users graphql): status - {response.status_code}: info - {str(response)}"
        )

        return response



import logging

from shikimori.types.message import MessageInfo
from .base import BaseEndpoint
from ..exceptions import RequestError

logger = logging.getLogger(__name__)


class MessageEndpoint(BaseEndpoint):
    async def ById(self, id: int) -> MessageInfo | RequestError:
        """
        Show a message.

        Requires messages oauth scope.

        :param id: must be a number
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/messages/{id}",
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return MessageInfo.from_dict(response)

        logger.debug(
            f"Bad Request(byId): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def create(
        self, from_id: int, to_id: int, body: str, kind: str = "Private"
    ) -> MessageInfo | RequestError:
        """
        Create a message.

        Requires messages oauth scope.

        :param from_id: Must be a number.
        :param to_id: Must be a number.
        :param body: Must be a String.
        :param kind: Must be one of: Private.
        """
        response = await self._request.make_request(
            "POST",
            url=f"{self._base_url}/api/messages/",
            json={
                "message": {
                    "from_id": from_id,
                    "to_id": to_id,
                    "body": body,
                    "kind": kind,
                }
            },
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return MessageInfo.from_dict(response)

        logger.debug(
            f"Bad Request(create): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def update(self, id: int, body: str) -> MessageInfo | RequestError:
        """
        Update a message.

        Requires messages oauth scope.

        :param id: must be a number
        :param body: must be a string
        """
        response = await self._request.make_request(
            "PATCH",
            url=f"{self._base_url}/api/messages/{id}",
            json={
                "message": {
                    "body": body,
                }
            },
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return MessageInfo.from_dict(response)

        logger.debug(
            f"Bad Request(update): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def delete(self, id: int) -> None | RequestError:
        """
        Destroy a message.

        Requires messages oauth scope.

        :param id: must be a number
        """
        response = await self._request.make_request(
            "DELETE",
            url=f"{self._base_url}/api/messages/{id}",
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return

        logger.debug(
            f"Bad Request(delete): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def markRead(self, ids: str, is_read: bool) -> None | RequestError:
        """
        mark as read or unread.

        Requires messages oauth scope.

        :param ids: string like '123, 34455, 283761846'
        :param is_read: True - read, False - Unread
        """
        response = await self._request.make_request(
            "POST",
            url=f"{self._base_url}/api/messages/mark_read",
            json={"ids": ids, "is_read": "1" if is_read else "0"},
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return

        logger.debug(
            f"Bad Request(mark_read): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def readAll(self, type: str) -> None | RequestError:
        """
        Requires messages oauth scope.

        :param type: Must be one of: news, notifications.
        """
        response = await self._request.make_request(
            "POST",
            url=f"{self._base_url}/api/messages/read_all",
            json={"type": type},
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return

        logger.debug(
            f"Bad Request(readAll): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def deleteAll(self, type: str) -> None | RequestError:
        """
        Requires messages oauth scope.

        :param type: Must be one of: news, notifications.
        """
        response = await self._request.make_request(
            "POST",
            url=f"{self._base_url}/api/messages/delete_all",
            json={"type": type},
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return

        logger.debug(
            f"Bad Request(deleteAll): status - {response.status_code}: info - {str(response)}"
        )

        return response



from dataclasses import dataclass

from shikimori.utils.filter import handle_none_data


@dataclass
class Topic:
    topic_id: str
    is_ignored: bool

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(topic_id=data.get("topic_id"), is_ignored=data.get("is_ignored"))



from dataclasses import dataclass
from .photo import Photo
from ..utils.filter import handle_none_data


@dataclass
class MangaRole:
    role: str
    roles: list[str]
    id: int
    name: str
    russian: str
    image: Photo
    url: str
    kind: str
    score: str
    status: str
    aired_on: str
    released_on: str
    volumes: int
    chapters: int

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            role=data.get("role"),
            roles=data.get("roles", []),
            volumes=data.get("volumes"),
            chapters=data.get("chapters"),
            id=data.get("id"),
            name=data.get("name"),
            russian=data.get("russian"),
            image=Photo.from_dict(data.get("image")),
            url=data.get("url"),
            kind=data.get("kind"),
            score=data.get("score"),
            status=data.get("status"),
            aired_on=data.get("aired_on"),
            released_on=data.get("released_on"),
        )


@dataclass
class AnimeRole:
    role: str
    roles: list[str]
    id: int
    name: str
    russian: str
    image: Photo
    url: str
    kind: str
    score: str
    status: str
    aired_on: str
    released_on: str
    episodes: int
    episodes_aired: int

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            role=data.get("role"),
            roles=data.get("roles", []),
            episodes=data.get("episodes"),
            episodes_aired=data.get("episodes_aired"),
            id=data.get("id"),
            name=data.get("name"),
            russian=data.get("russian"),
            image=Photo.from_dict(data.get("image")),
            url=data.get("url"),
            kind=data.get("kind"),
            score=data.get("score"),
            status=data.get("status"),
            aired_on=data.get("aired_on"),
            released_on=data.get("released_on"),
        )


@dataclass
class CharacterBrief:
    id: int
    name: str
    russian: str
    image: Photo
    url: str

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            id=data.get("id"),
            name=data.get("name"),
            russian=data.get("russian"),
            image=Photo.from_dict(data.get("image")),
            url=data.get("url"),
        )


@dataclass
class Character:
    id: int
    name: str
    russian: str
    image: Photo
    url: str
    altname: str
    japanese: str
    description: str
    description_html: str
    description_source: str
    favoured: bool
    thread_id: int
    topic_id: int
    updated_at: str
    seyu: list[CharacterBrief]
    animes: list[AnimeRole]
    mangas: list[MangaRole]

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            altname=data.get("altname"),
            japanese=data.get("japanese"),
            description=data.get("description"),
            description_html=data.get("description_html"),
            description_source=data.get("description_source"),
            favoured=data.get("favoured"),
            thread_id=data.get("thread_id"),
            topic_id=data.get("topic_id"),
            updated_at=data.get("updated_at"),
            seyu=[
                CharacterBrief.from_dict(seyu_data)
                for seyu_data in data.get("seyu", [])
            ],
            animes=[
                AnimeRole.from_dict(anime_data) for anime_data in data.get("animes", [])
            ],
            mangas=[
                MangaRole.from_dict(manga_data) for manga_data in data.get("mangas", [])
            ],
            id=data.get("id"),
            name=data.get("name"),
            russian=data.get("russian"),
            image=Photo.from_dict(data.get("image")),
            url=data.get("url"),
        )



import logging

from shikimori.types.review import Review
from .base import BaseEndpoint
from ..exceptions import RequestError
from ..utils.filter import filter_none_parameters

logger = logging.getLogger(__name__)


class ReviewEndpoint(BaseEndpoint):
    async def create(
        self, anime_id: int, body: str, opinion: str
    ) -> Review | RequestError:
        """
        Create a review.

        :param anime_id: Must be a number.
        :param body: Must be a String
        :param opinion: Must be one of: positive, neutral, negative
        """
        response = await self._request.make_request(
            "POST",
            url=f"{self._base_url}/api/reviews",
            json={
                "review": filter_none_parameters(
                    {"anime_id": anime_id, "body": body, "opinion": opinion}
                )
            },
        )

        if not isinstance(response, RequestError):
            return Review.from_dict(response)

        logger.debug(
            f"Bad Request(create): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def update(self, id: int, body: str, opinion: str) -> Review | RequestError:
        """
        Update a review.

        :param id: Must be a number.
        :param body: Must be a String
        :param opinion: Must be one of: positive, neutral, negative
        """
        response = await self._request.make_request(
            "PATCH",
            headers=self.headers,
            url=f"{self._base_url}/api/reviews/{id}",
            json={"review": filter_none_parameters({"body": body, "opinion": opinion})},
        )

        if not isinstance(response, RequestError):
            return Review.from_dict(response)

        logger.debug(
            f"Bad Request(update): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def delete(self, id: int) -> Review | RequestError:
        """
        Destroy a review.

        :param id: Must be a number.
        """
        response = await self._request.make_request(
            "DELETE",
            headers=self.headers,
            url=f"{self._base_url}/api/reviews/{id}",
        )

        if not isinstance(response, RequestError):
            return response["notice"]

        logger.debug(
            f"Bad Request(delete): status - {response.status_code}: info - {str(response)}"
        )

        return response



import logging

from shikimori.types.topic_ignore import Topic
from .base import BaseEndpoint
from ..exceptions import RequestError

logger = logging.getLogger(__name__)


class TopicIgnoreEndpoint(BaseEndpoint):
    async def ignore(self, topic_id: str | int) -> Topic | RequestError:
        """
        ignore topic.

        Requires topics oauth scope.

        :param topic_id: Must be a number
        """

        response = await self._request.make_request(
            "POST",
            url=f"{self._base_url}/api/v2/topics/{topic_id}/ignore",
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return Topic.from_dict(response)

        logger.debug(
            f"Bad Request(ignore): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def unignore(
        self,
        topic_id: str | int,
    ) -> Topic | RequestError:
        """
        unignore topic.

        Requires topics oauth scope.

        :param topic_id: must be a number
        """

        response = await self._request.make_request(
            "DELETE",
            url=f"{self._base_url}/api/v2/topics/{topic_id}/ignore",
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return Topic.from_dict(response)

        logger.debug(
            f"Bad Request(unignore): status - {response.status_code}: info - {str(response)}"
        )

        return response



import logging
from typing import List

from shikimori.types.studios import Studio
from .base import BaseEndpoint
from ..exceptions import RequestError

logger = logging.getLogger(__name__)


class StudiosEndpoint(BaseEndpoint):
    async def list(self) -> List[Studio] | RequestError:
        """
        List studios
        """
        response = await self._request.make_request(
            "GET", url=f"{self._base_url}/api/studios", headers=self.headers
        )

        if not isinstance(response, RequestError):
            return [Studio.from_dict(studio) for studio in response]

        logger.debug(
            f"Bad Request(list): status - {response.status_code}: info - {str(response)}"
        )

        return response



import logging

from shikimori.types.photo import UserImage
from .base import BaseEndpoint
from ..exceptions import RequestError
from ..utils.filter import filter_none_parameters

logger = logging.getLogger(__name__)


class UserImageEndpoint(BaseEndpoint):
    async def upload(
        self, image: str, linked_type: str = None
    ) -> UserImage | RequestError:
        """
        Create a user image.

        Requires comments oauth scope.

        :param image: Must be a String
        :param linked_type: Must be a String
        """

        response = await self._request.make_request(
            "POST",
            url=f"{self._base_url}/api/user_images",
            headers=self.headers,
            json=filter_none_parameters({"image": image, "linked_type": linked_type}),
        )

        if not isinstance(response, RequestError):
            return UserImage.from_dict(response)

        logger.debug(
            f"Bad Request(upload): status - {response.status_code}: info - {str(response)}"
        )

        return response



import logging

from shikimori.types.character import Character, CharacterBrief
from .base import BaseEndpoint
from ..exceptions import RequestError

logger = logging.getLogger(__name__)


class CharacterEndpoint(BaseEndpoint):
    async def ById(self, id: int) -> Character | RequestError:
        """
        Show a character.

        :param id: must be a number.
        """

        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/characters/{id}",
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return Character.from_dict(response)

        logger.debug(
            f"Bad Request(ById): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def search(self, search: str) -> list[CharacterBrief] | RequestError:
        """
        Search characters.

        :param search: Must be a String.
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/characters/search",
            params={"search": search},
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return [CharacterBrief.from_dict(ch) for ch in response]

        logger.debug(
            f"Bad Request(search): status - {response.status_code}: info - {str(response)}"
        )

        return response
